"""AliceBot 协议适配器。

所有协议适配器都必须继承自 `Adapter` 基类。
"""
import os
import asyncio
from abc import ABC, abstractmethod
from typing import TYPE_CHECKING, Union, Callable, Optional, Awaitable

from alicebot.typing import T_Event
from alicebot.log import error_or_exception
from alicebot.utils import sync_func_wrapper

if TYPE_CHECKING:
    from alicebot import Bot

__all__ = ["Adapter"]

if os.getenv("ALICEBOT_DEV") == "1":
    # 当处于开发环境时，使用 pkg_resources 风格的命名空间包
    __import__("pkg_resources").declare_namespace(__name__)


class Adapter(ABC):
    """协议适配器基类。

    Attributes:
        name: 适配器的名称。
        bot: 当前的机器人对象。
    """

    name: str
    bot: "Bot"

    def __init__(self, bot: "Bot"):
        if not hasattr(self, "name"):
            self.name = self.__class__.__name__
        self.bot: "Bot" = bot
        self.handle_event = self.bot.handle_event

    async def safe_run(self):
        """附带有异常处理地安全运行适配器。"""
        try:
            await self.run()
        except Exception as e:
            error_or_exception(
                f"Run adapter {self.__class__.__name__} failed:",
                e,
                self.bot.config.verbose_exception_log,
            )

    @abstractmethod
    async def run(self):
        """适配器运行方法，适配器开发者必须实现该方法。

        适配器运行过程中保持保持运行，当此方法结束后，AliceBot 不会自动重新启动适配器。
        """
        raise NotImplementedError

    async def startup(self):
        """在适配器开始运行前运行的方法，用于初始化适配器。

        AliceBot 依次运行并等待所有适配器的 `startup()` 方法，待运行完毕后再创建 `run()` 任务。
        """
        pass

    async def shutdown(self):
        """在适配器结束运行时运行的方法，用于安全地关闭适配器。

        AliceBot 在接收到系统的结束信号后依次运行并等待所有适配器的 `shutdown()` 方法。
        当强制退出时此方法可能未被执行。
        """
        pass

    async def send(self, *args, **kwargs):
        """发送消息，需要适配器开发者实现。"""
        raise NotImplementedError

    async def get(
        self,
        func: Optional[Callable[[T_Event], Union[bool, Awaitable[bool]]]] = None,
        *,
        max_try_times: Optional[int] = None,
        timeout: Optional[Union[int, float]] = None,
    ) -> T_Event:
        """获取满足指定条件的的事件，协程会等待直到适配器接收到满足条件的事件、超过最大事件数或超时。

        类似 Bot 类的 get() 方法，但是隐含了判断产生 event 的适配器是本适配器。

        Args:
            func: 协程或者函数，函数会被自动包装为协程执行。
                要求接受一个事件作为参数，返回布尔值。
                当协程返回 `True` 时返回当前事件。
                当为 `None` 时相当于输入对于任何事件均返回真的协程，即返回适配器接收到的下一个事件。
            max_try_times: 最大事件数。
            timeout: 超时时间。

        Returns:
            返回满足 func 条件的事件。

        Raises:
            GetEventTimeout: 超过最大事件数或超时。
        """

        def func_wrapper(_func):
            async def _wrapper(_event: T_Event):
                if _event.adapter is not self:
                    return False
                return await _func(_event)

            return _wrapper

        if func is None:
            func = sync_func_wrapper(lambda x: True)
        elif not asyncio.iscoroutinefunction(func):
            func = sync_func_wrapper(func)
        func = func_wrapper(func)

        return await self.bot.get(func, max_try_times=max_try_times, timeout=timeout)



import os
import json
import asyncio
import inspect
import os.path
import pkgutil
import importlib
import traceback
import dataclasses
from abc import ABC
from types import ModuleType
from functools import partial
from typing_extensions import ParamSpec
from importlib.abc import MetaPathFinder
from importlib.machinery import PathFinder
from typing import List, Type, Generic, TypeVar, Callable, Iterable, Optional, Coroutine

from pydantic import BaseModel

from alicebot.exceptions import LoadModuleError

__all__ = [
    "ModulePathFinder",
    "ModuleInfo",
    "load_module",
    "load_module_from_name",
    "load_modules_from_dir",
    "DataclassEncoder",
    "samefile",
    "get_module_name",
    "sync_func_wrapper",
]

_T = TypeVar("_T")
_P = ParamSpec("_P")
_R = TypeVar("_R")


class ModulePathFinder(MetaPathFinder):
    """用于查找 AliceBot 组件的元路径查找器。"""

    path: List[str] = []

    def find_spec(self, fullname, path=None, target=None):
        if path is None:
            path = []
        return PathFinder.find_spec(fullname, self.path + list(path), target)


class ModuleInfo(Generic[_T]):
    """模块信息。

    Attributes
        module_class: 模块类。
        config_class: 配置类。
        module: 模块。
    """

    __slots__ = ("module_class", "config_class", "module")

    def __init__(
        self,
        module_class: Type[_T],
        config_class: Optional[Type[BaseModel]],
        module: ModuleType,
    ):
        self.module_class = module_class
        self.config_class = config_class
        self.module = module


def load_module(module: ModuleType, class_type: Type[_T]) -> ModuleInfo[_T]:
    """从模块中查找指定类型的类和 `Config` 。若模块中存在多个符合条件的类，则抛出错误。

    Args:
        module: Python 模块。
        class_type: 要查找的类型。

    Returns:
        返回符合条件的类和配置类组成的元组。

    Raises:
        LoadModuleError: 当找不到符合条件的类或者模块中存在多个符合条件的类。
    """

    module_class: Optional[Type[_T]] = None
    for _, module_attr in inspect.getmembers(module, inspect.isclass):
        module_attr: type
        if (
            (inspect.getmodule(module_attr) or module) is module
            and issubclass(module_attr, class_type)
            and module_attr != class_type
            and ABC not in module_attr.__bases__
            and not inspect.isabstract(module_attr)
        ):
            if module_class is None:
                module_class = module_attr
            else:
                raise LoadModuleError(
                    f"More then one {class_type!r} class "
                    f"in the {module.__name__} module"
                )
    if module_class is None:
        raise LoadModuleError(
            f"Can not find {class_type!r} class in the {module.__name__} module"
        )

    config_class: Optional[Type[BaseModel]] = None
    for name, module_attr in inspect.getmembers(module, inspect.isclass):
        module_attr: type
        if (
            name == "Config"
            and issubclass(module_attr, BaseModel)
            and isinstance(getattr(module_attr, "__config_name__", None), str)
        ):
            config_class = module_attr

    return ModuleInfo(module_class, config_class, module)


def load_module_from_name(name: str, class_type: Type[_T]) -> ModuleInfo[_T]:
    """从模块中查找指定类型的类和 `Config` 。若模块中存在多个符合条件的类，则抛出错误。

    Args:
        name: 模块名称，格式和 Python `import` 语句相同。
        class_type: 要查找的类型。

    Returns:
        返回符合条件的 `ModuleInfo`。

    Raises:
        ImportError: 当导入模块过程中出现错误。
        LoadModuleError: 当找不到符合条件的类或者模块中存在多个符合条件的类。
    """
    try:
        importlib.invalidate_caches()
        module = importlib.import_module(name)
        importlib.reload(module)
        return load_module(module, class_type)
    except LoadModuleError as e:
        raise e
    except BaseException as e:
        # 不捕获 KeyboardInterrupt
        # 捕获 KeyboardInterrupt 会阻止用户关闭 Python 当正在导入的模块陷入死循环时
        if isinstance(e, KeyboardInterrupt):
            raise e
        raise ImportError(e, traceback.format_exc()) from e


def load_modules_from_dir(
    path: Iterable[str], class_type: Type[_T]
) -> List[ModuleInfo[_T]]:
    """从指定路径列表中的所有模块中查找指定类型的类和 `Config` ，以 `_` 开头的插件不会被导入。路径可以是相对路径或绝对路径。

    Args:
        path: 由储存模块的路径文本组成的列表。 例如 `['path/of/plugins/', '/home/xxx/alicebot/plugins']` 。
        class_type: 要查找的类型。

    Returns:
        返回符合条件的 `ModuleInfo` 的列表。
    """
    modules = []
    for module_info in pkgutil.iter_modules(path):
        if not module_info.name.startswith("_"):
            try:
                modules.append(load_module_from_name(module_info.name, class_type))
            except (ImportError, LoadModuleError):
                continue
    return modules


class DataclassEncoder(json.JSONEncoder):
    """用于解析 MessageSegment 的 JSONEncoder 类。"""

    def default(self, o):
        if dataclasses.is_dataclass(o):
            return o.as_dict()
        return super().default(o)


def samefile(path1: str, path2: str) -> bool:
    """一个 `os.path.samefile` 的简单包装。

    Args:
        path1: 路径1。
        path2: 路径2。

    Returns:
        如果两个路径是否指向相同的文件或目录。
    """
    try:
        return path1 == path2 or os.path.samefile(path1, path2)
    except OSError:
        return False


def get_module_name(path: str) -> str:
    """
    从路径中获取合法的 Python 模块名称，与 `inspect.getmodulename()` 函数不同。

    Args:
        path: 路径。

    Returns:
        Python 模块名称。

    Raises:
        ValueError: 当路径不以 ".py" 结尾时。
    """
    basename = os.path.basename(path)
    if not basename.endswith(".py"):
        raise ValueError('path must endswith ".py"')
    if basename == "__init__.py":
        return os.path.basename(os.path.dirname(path))
    return basename[:-3]


def sync_func_wrapper(
    func: Callable[_P, _R], to_thread: bool = False
) -> Callable[_P, Coroutine[None, None, _R]]:
    """包装一个同步函数为异步函数，当 func 为

    Args:
        func: 待包装的同步函数。
        to_thread: 在独立的线程中运行同步函数。

    Returns:
        异步函数。
    """
    if to_thread:

        async def _wrapper(*args: _P.args, **kwargs: _P.kwargs):
            loop = asyncio.get_running_loop()
            func_call = partial(func, *args, **kwargs)
            return await loop.run_in_executor(None, func_call)

    else:

        async def _wrapper(*args: _P.args, **kwargs: _P.kwargs):
            return func(*args, **kwargs)

    return _wrapper



"""AliceBot 配置。

AliceBot 使用 [pydantic](https://pydantic-docs.helpmanual.io/) 来读取配置。
"""
from typing import Set

from pydantic import Field, BaseModel

__all__ = ["MainConfig"]


class MainConfig(BaseModel):
    """AliceBot 的主要配置。

    Attributes:
        plugins: 将被加载的插件列表，将依次被 `Bot` 类的 `load_plugin()` 方法加载。
        plugin_dir: 将被加载的插件目录列表，将被 `Bot` 类的 `load_plugins_from_dir()` 方法加载。
        adapters: 将被加载的适配器列表，将依次被 `Bot` 类的 `load_adapter()` 方法加载。
        verbose_exception_log: 详细的异常记录，设置为 True 时会在日志中添加异常的 Traceback。
    """

    plugins: Set[str] = Field(default_factory=set)
    plugin_dir: Set[str] = Field(default_factory=set)
    adapters: Set[str] = Field(default_factory=set)
    verbose_exception_log: bool = False

    class Config:
        extra = "allow"



"""测试适配器。

这里是一个最简单可以直接使用的适配器示例。
"""
from asyncio import Condition

from alicebot.event import Event
from alicebot.adapter import Adapter


class TestAdapter(Adapter):
    """测试适配器。"""

    name: str = "test"
    _msg: str = ""
    _cond: Condition

    async def startup(self):
        self._cond = Condition()

    async def run(self):
        while not self.bot.should_exit.is_set():
            async with self._cond:
                await self._cond.wait()
                await self.handle_event(
                    TestAdapterEvent(adapter=self, type="message", message=self._msg)
                )

    async def send(self, msg: str):
        """此方法发送的消息会直接使此适配器产生一个事件。"""
        async with self._cond:
            self._msg = msg
            self._cond.notify()


class TestAdapterEvent(Event[TestAdapter]):
    """测试适配器事件。

    Attributes:
        message: 消息内容。
    """

    message: str



"""AliceBot 类型提示支持。

此模块定义了部分 AliceBot 使用的类型。
"""

from typing import TYPE_CHECKING, TypeVar, Callable, NoReturn, Awaitable

from alicebot.message import T_MS, T_Message, T_MessageSegment

if TYPE_CHECKING:
    from alicebot import Bot  # noqa
    from alicebot.event import Event  # noqa
    from alicebot.plugin import Plugin  # noqa
    from alicebot.adapter import Adapter  # noqa

__all__ = [
    "T_Event",
    "T_Plugin",
    "T_Adapter",
    "T_Message",
    "T_MessageSegment",
    "T_MS",
    "T_BotHook",
    "T_BotExitHook",
    "T_AdapterHook",
    "T_EventHook",
    "T_State",
]

T_Event = TypeVar("T_Event", bound="Event")
T_Plugin = TypeVar("T_Plugin", bound="Plugin")
T_Adapter = TypeVar("T_Adapter", bound="Adapter")
T_State = TypeVar("T_State")

T_BotHook = Callable[["Bot"], Awaitable[NoReturn]]
T_BotExitHook = Callable[["Bot"], NoReturn]
T_AdapterHook = Callable[[T_Adapter], Awaitable[NoReturn]]
T_EventHook = Callable[[T_Event], Awaitable[NoReturn]]



import os
import sys
import json
import time
import signal
import asyncio
import threading
from itertools import chain
from collections import defaultdict
from typing import (
    Any,
    Dict,
    List,
    Type,
    Tuple,
    Union,
    Callable,
    Iterable,
    Optional,
    Awaitable,
)

from pydantic import BaseModel, ValidationError, create_model

from alicebot.plugin import Plugin
from alicebot.adapter import Adapter
from alicebot.config import MainConfig
from alicebot.log import logger, error_or_exception
from alicebot.typing import (
    T_Event,
    T_BotHook,
    T_EventHook,
    T_AdapterHook,
    T_BotExitHook,
)
from alicebot.exceptions import (
    SkipException,
    StopException,
    GetEventTimeout,
    LoadModuleError,
)
from alicebot.utils import (
    ModuleInfo,
    ModulePathFinder,
    samefile,
    get_module_name,
    sync_func_wrapper,
    load_module_from_name,
    load_modules_from_dir,
)

__all__ = ["Bot"]

HANDLED_SIGNALS = (
    signal.SIGINT,  # Unix signal 2. Sent by Ctrl+C.
    signal.SIGTERM,  # Unix signal 15. Sent by `kill <pid>`.
)


class Bot:
    """AliceBot 机器人对象，定义了机器人的基本方法。
        读取并储存配置 `Config`，加载适配器 `Adapter` 和插件 `Plugin`，并进行事件分发。

    Attributes:
        config: 机器人配置。
        should_exit: 机器人是否应该进入准备退出状态。
        plugins_priority_dict: 插件优先级字典。
        plugin_state: 插件状态。
        global_state: 全局状态。
    """

    config: MainConfig
    should_exit: asyncio.Event
    plugins_priority_dict: Dict[int, List[ModuleInfo]]
    plugin_state: Dict[str, Any]
    global_state: Dict[Any, Any]

    _adapters: List[Tuple[Adapter, ModuleInfo]]  # 适配器和适配器信息列表
    _condition: asyncio.Condition  # 用于处理 get 的 Condition
    _current_event: T_Event  # 当前待处理的 Event

    _restart_flag: bool  # 重新启动标志
    _module_path_finder: ModulePathFinder  # 用于查找 plugins 的模块元路径查找器
    _raw_config_dict: Dict[str, Any]  # 原始配置字典
    _config_update_dict: Dict[str, Tuple[Type[BaseModel], Any]]  # 配置模型更新使用的字典

    _config_file: Optional[str]  # 配置文件
    _config_dict: Optional[Dict[str, Any]]  # 配置字典
    _hot_reload: bool  # 热重载

    _extend_plugins: List[str]  # 使用 load_plugin() 方法程序化加载的插件列表
    _extend_plugin_dirs: List[str]  # 使用 load_plugins_from_dir() 方法程序化加载的插件路径列表
    _extend_adapters: List[str]  # 使用 load_adapter() 方法程序化加载的适配器列表
    _bot_run_hooks: List[T_BotHook]
    _bot_exit_hooks: List[T_BotExitHook]
    _adapter_startup_hooks: List[T_AdapterHook]
    _adapter_run_hooks: List[T_AdapterHook]
    _adapter_shutdown_hooks: List[T_AdapterHook]
    _event_preprocessor_hooks: List[T_EventHook]
    _event_postprocessor_hooks: List[T_EventHook]

    def __init__(
        self,
        *,
        config_file: Optional[str] = "config.json",
        config_dict: Optional[Dict] = None,
        hot_reload: bool = False,
    ):
        """初始化 AliceBot ，读取配置文件，创建配置，加载适配器和插件。

        Args:
            config_file: 配置文件，如不指定则使用默认的 `config.json`。
                若指定为 None，则不加载配置文件。
            config_dict: 配置字典，默认为 None。
                若指定字典，则会忽略 config_file 配置，不再读取配置文件。
            hot_reload: 热重载。
                启用后将自动检查 `plugin_dir` 中的插件文件更新，并在更新时自动重新加载。
        """
        self.config = MainConfig()
        self.plugins_priority_dict = defaultdict(list)
        self.plugin_state = defaultdict(type(None))
        self.global_state = {}

        self._adapters = []
        self._restart_flag = False
        self._module_path_finder = ModulePathFinder()
        self._raw_config_dict = {}
        self._config_update_dict = {}

        self._config_file = config_file
        self._config_dict = config_dict
        self._hot_reload = hot_reload

        self._extend_plugins = []
        self._extend_plugin_dirs = []
        self._extend_adapters = []
        self._bot_run_hooks = []
        self._bot_exit_hooks = []
        self._adapter_startup_hooks = []
        self._adapter_run_hooks = []
        self._adapter_shutdown_hooks = []
        self._event_preprocessor_hooks = []
        self._event_postprocessor_hooks = []

        sys.meta_path.insert(0, self._module_path_finder)

    @property
    def plugins(self) -> List[Type[Plugin]]:
        """当前已经加载的插件的列表。"""
        return list(
            map(
                lambda x: x.module_class,
                chain(*self.plugins_priority_dict.values()),
            )
        )

    @property
    def adapters(self) -> List[Adapter]:
        """当前已经加载的适配器的列表。"""
        return list(map(lambda x: x[0], self._adapters))

    def run(self):
        """运行 AliceBot，监听并拦截系统退出信号，更新机器人配置。"""
        self._restart_flag = True
        while self._restart_flag:
            self._restart_flag = False
            asyncio.run(self._run())
            if self._restart_flag:
                self._load_plugins_from_dir(self._extend_plugin_dirs)
                for _plugin in self._extend_plugins:
                    self._load_plugin(_plugin)
                for _adapter in self._extend_adapters:
                    self._load_adapter(_adapter)

    def restart(self):
        """退出并重新运行 AliceBot。"""
        logger.info("Restarting AliceBot...")
        self._restart_flag = True
        self.should_exit.set()

    async def _run(self):
        """运行 AliceBot。"""
        self.should_exit = asyncio.Event()
        self._condition = asyncio.Condition()

        # 监听并拦截系统退出信号，从而完成一些善后工作后再关闭程序
        if threading.current_thread() is threading.main_thread():
            # Signal 仅能在主线程中被处理。
            try:
                loop = asyncio.get_running_loop()
                for sig in HANDLED_SIGNALS:
                    loop.add_signal_handler(sig, self._handle_exit)
            except NotImplementedError:
                # add_signal_handler 仅在 Unix 下可用，以下对于 Windows。
                for sig in HANDLED_SIGNALS:
                    signal.signal(sig, self._handle_exit)

        # 加载配置文件
        self._raw_config_dict = {}
        if self._config_dict is not None:
            self._raw_config_dict = self._config_dict
        elif self._config_file is not None:
            try:
                with open(self._config_file, "r", encoding="utf8") as f:
                    self._raw_config_dict = json.load(f)
            except OSError as e:
                logger.warning(f"Can not open config file: {e!r}")
            except json.JSONDecodeError as e:
                logger.warning(f"Read config file failed: {e!r}")
            except ValueError as e:
                error_or_exception(
                    "Read config file failed:", e, self.config.verbose_exception_log
                )

        try:
            self.config = MainConfig(**self._raw_config_dict)
        except ValidationError as e:
            self.config = MainConfig()
            error_or_exception(
                "Config dict parse error:", e, self.config.verbose_exception_log
            )

        self._load_plugins_from_dir(self.config.plugin_dir)
        for _plugin in self.config.plugins:
            self._load_plugin(_plugin)
        for _adapter in self.config.adapters:
            self._load_adapter(_adapter)
        self._reload_config()

        # 启动 AliceBot
        logger.info("Running AliceBot...")

        hot_reload_task = None
        if self._hot_reload:
            hot_reload_task = asyncio.create_task(self._run_hot_reload())

        for _hook_func in self._bot_run_hooks:
            await _hook_func(self)

        try:
            for _adapter, _ in self._adapters:
                for _hook_func in self._adapter_startup_hooks:
                    await _hook_func(_adapter)
                try:
                    await _adapter.startup()
                except Exception as e:
                    error_or_exception(
                        f"Startup adapter {_adapter!r} failed:",
                        e,
                        self.config.verbose_exception_log,
                    )

            for _adapter, _ in self._adapters:
                for _hook_func in self._adapter_run_hooks:
                    await _hook_func(_adapter)
                asyncio.create_task(_adapter.safe_run())

            await self.should_exit.wait()

            if hot_reload_task is not None:
                await hot_reload_task
        finally:
            for _adapter, _ in self._adapters:
                for _hook_func in self._adapter_shutdown_hooks:
                    await _hook_func(_adapter)
                await _adapter.shutdown()
            for _hook_func in self._bot_exit_hooks:
                _hook_func(self)

            self._adapters.clear()
            self.plugins_priority_dict.clear()
            self._config_update_dict.clear()
            self._module_path_finder.path.clear()

    def _remove_plugin_by_path(self, file: str) -> Optional[ModuleInfo]:
        for plugins in self.plugins_priority_dict.values():
            for i, _plugin in enumerate(plugins):
                if samefile(getattr(_plugin.module, "__file__", None), file):
                    plugins.pop(i)
                    self._remove_config_module(_plugin.config_class)
                    return _plugin
        return None

    async def _run_hot_reload(self):
        """热重载。"""
        try:
            from watchfiles import Change, PythonFilter, DefaultFilter, awatch
        except ImportError:
            logger.warning(
                'Hot reload needs to install "watchfiles", '
                'try "pip install watchfiles"'
            )
            return

        logger.info("Hot reload is working!")
        async for changes in awatch(
            *map(
                os.path.abspath,
                self.config.plugin_dir.union(
                    {self._config_file}
                    if self._config_dict is None and self._config_file is not None
                    else set()
                ),
            ),
            stop_event=self.should_exit,
        ):
            # 按照 Change.deleted, Change.modified, Change.added 的顺序处理
            # 以确保发生重命名时先处理删除再处理新增
            for change_type, file in sorted(changes, key=lambda x: x[0], reverse=True):
                # 更改配置文件
                if (
                    samefile(self._config_file, file)
                    and change_type == change_type.modified
                ):
                    logger.info(f'Reload config file "{self._config_file}"')
                    self.restart()
                    continue

                # 更改插件文件夹
                if change_type == Change.deleted:
                    # 针对删除操作特殊处理
                    if not file.endswith(".py"):
                        file = os.path.join(file, "__init__.py")
                else:
                    if os.path.isdir(file) and os.path.isfile(
                        os.path.join(file, "__init__.py")
                    ):
                        # 当新增一个目录，且此目录中包含 __init__.py 文件
                        # 说明此时发生的是添加一个 Python 包，则视为添加了此包的 __init__.py 文件
                        file = os.path.join(file, "__init__.py")
                    if not (os.path.isfile(file) and file.endswith(".py")):
                        continue

                if change_type == Change.added:
                    try:
                        plugin_info = load_module_from_name(
                            get_module_name(file), Plugin
                        )
                        self._load_plugin_module(plugin_info)
                        self._reload_config()
                    except Exception as e:
                        error_or_exception(
                            f'Add new plugin from file "{file}" failed:',
                            e,
                            self.config.verbose_exception_log,
                        )
                    else:
                        logger.info(
                            "Succeeded to add new plugin "
                            f'"{plugin_info.module_class.__name__}" from file "{file}"'
                        )
                    continue
                elif change_type == Change.deleted:
                    try:
                        _plugin = self._remove_plugin_by_path(file)
                        self._reload_config()
                    except Exception as e:
                        error_or_exception(
                            f'Removed plugin from file "{file}" failed:',
                            e,
                            self.config.verbose_exception_log,
                        )
                    else:
                        if _plugin is not None:
                            logger.info(
                                "Succeeded to remove plugin "
                                f'"{_plugin.module_class.__name__}" from file "{file}"'
                            )
                elif change_type == Change.modified:
                    msg = "reload"
                    try:
                        _plugin = self._remove_plugin_by_path(file)
                        if _plugin is not None:
                            plugin_module_name = _plugin.module.__name__
                        else:
                            msg = "add new"
                            plugin_module_name = get_module_name(file)
                        plugin_info = load_module_from_name(plugin_module_name, Plugin)
                        self._load_plugin_module(plugin_info)
                        self._reload_config()
                    except Exception as e:
                        error_or_exception(
                            f'{msg.capitalize()} plugin from file "{file}" failed:',
                            e,
                            self.config.verbose_exception_log,
                        )
                    else:
                        logger.info(
                            f"Succeeded to {msg} plugin "
                            f'"{plugin_info.module_class.__name__}" from file "{file}"'
                        )

    def reload_plugins(self):
        """手动重新加载所有插件。"""
        self.plugins_priority_dict.clear()
        self._config_update_dict.clear()
        self._load_plugins_from_dir(self.config.plugin_dir)
        for _plugin in self.config.plugins:
            self._load_plugin(_plugin)
        self._load_plugins_from_dir(self._extend_plugin_dirs)
        for _plugin in self._extend_plugins:
            self._load_plugin(_plugin)
        for _, adapter_info in self._adapters:
            self._update_config_module(adapter_info.config_class)
        self._reload_config()

    def _handle_exit(self, *args):  # noqa
        """当机器人收到退出信号时，根据情况进行处理。"""
        logger.info("Stopping AliceBot...")
        if self.should_exit.is_set():
            logger.warning("Force Exit AliceBot...")
            sys.exit()
        else:
            self.should_exit.set()

    async def handle_event(
        self, current_event: T_Event, *, handle_get: bool = True, show_log: bool = True
    ):
        """被适配器对象调用，根据优先级分发事件给所有插件，并处理插件的 `stop` 、 `skip` 等信号。

        此方法不应该被用户手动调用。

        Args:
            current_event: 当前待处理的 `Event`。
            handle_get: 当前事件是否可以被 get 方法捕获，默认为 True。
            show_log: 是否在日志中显示，默认为 True。
        """
        if show_log:
            logger.info(
                f"Adapter {current_event.adapter.name} received: {current_event!r}"
            )

        if handle_get:
            asyncio.create_task(self._handle_event())
            await asyncio.sleep(0)
            async with self._condition:
                self._current_event = current_event
                self._condition.notify_all()
        else:
            asyncio.create_task(self._handle_event(current_event))

    async def _handle_event(self, current_event: Optional[T_Event] = None):
        if current_event is None:
            async with self._condition:
                await self._condition.wait()
                current_event = self._current_event
            if current_event.__handled__:
                return

        for _hook_func in self._event_preprocessor_hooks:
            await _hook_func(current_event)

        for plugin_priority in sorted(self.plugins_priority_dict.keys()):
            try:
                logger.debug(
                    f"Checking for matching plugins with priority {plugin_priority!r}"
                )
                stop = False
                for _plugin in self.plugins_priority_dict[plugin_priority]:
                    try:
                        _plugin = _plugin.module_class(current_event)
                        if await _plugin.rule():
                            logger.info(f"Event will be handled by {_plugin!r}")
                            try:
                                await _plugin.handle()
                            finally:
                                if _plugin.block:
                                    stop = True
                    except SkipException:
                        # 插件要求跳过自身继续当前事件传播
                        continue
                    except StopException:
                        # 插件要求停止当前事件传播
                        stop = True
                    except Exception as e:
                        error_or_exception(
                            f'Exception in plugin "{_plugin}":',
                            e,
                            self.config.verbose_exception_log,
                        )
                if stop:
                    break
            except Exception as e:
                error_or_exception(
                    f"Exception in handling event {current_event!r}:",
                    e,
                    self.config.verbose_exception_log,
                )

        for _hook_func in self._event_postprocessor_hooks:
            await _hook_func(current_event)

        logger.info("Event Finished")

    async def get(
        self,
        func: Optional[Callable[[T_Event], Union[bool, Awaitable[bool]]]] = None,
        *,
        max_try_times: Optional[int] = None,
        timeout: Optional[Union[int, float]] = None,
    ) -> T_Event:
        """获取满足指定条件的的事件，协程会等待直到适配器接收到满足条件的事件、超过最大事件数或超时。

        Args:
            func: 协程或者函数，函数会被自动包装为协程执行。
                要求接受一个事件作为参数，返回布尔值。当协程返回 `True` 时返回当前事件。
                当为 `None` 时相当于输入对于任何事件均返回真的协程，即返回适配器接收到的下一个事件。
            max_try_times: 最大事件数。
            timeout: 超时时间。

        Returns:
            返回满足 func 条件的事件。

        Raises:
            GetEventTimeout: 超过最大事件数或超时。
        """
        if func is None:
            func = sync_func_wrapper(lambda x: True)
        elif not asyncio.iscoroutinefunction(func):
            func = sync_func_wrapper(func)

        try_times = 0
        start_time = time.time()
        while not self.should_exit.is_set():
            if max_try_times is not None and try_times > max_try_times:
                break
            if timeout is not None and time.time() - start_time > timeout:
                break

            async with self._condition:
                if timeout is None:
                    await self._condition.wait()
                else:
                    try:
                        await asyncio.wait_for(
                            self._condition.wait(),
                            timeout=start_time + timeout - time.time(),
                        )
                    except asyncio.TimeoutError:
                        break

                if not self._current_event.__handled__:
                    if await func(self._current_event):
                        self._current_event.__handled__ = True
                        return self._current_event

                try_times += 1

        if not self.should_exit.is_set():
            raise GetEventTimeout

    def _load_plugin_module(self, plugin_info: ModuleInfo):
        """加载插件。"""
        priority = getattr(plugin_info.module_class, "priority", None)
        if type(priority) is int and priority >= 0:
            for _plugin in self.plugins:
                if _plugin.__name__ == plugin_info.module_class.__name__:
                    logger.warning(
                        f'Already have a same name plugin "{_plugin.__name__}"'
                    )
            self.plugins_priority_dict[priority].append(plugin_info)
            self._update_config_module(plugin_info.config_class)
        else:
            raise LoadModuleError(
                "Plugin class priority incorrect in the module "
                + plugin_info.module.__name__
            )

    def _load_plugin(self, name: str) -> Optional[Type[Plugin]]:
        """加载单个插件。

        Args:
            name: 插件名称，格式和 Python `import` 语句相同。

        Returns:
            被加载的插件类。
        """
        try:
            plugin_info = load_module_from_name(name, Plugin)
            self._load_plugin_module(plugin_info)
        except Exception as e:
            error_or_exception(
                f'Load plugin from module "{name}" failed:',
                e,
                self.config.verbose_exception_log,
            )
        else:
            logger.info(
                f'Succeeded to load plugin "{plugin_info.module_class.__name__}" '
                f'from module "{name}"'
            )
            return plugin_info.module_class

    def load_plugin(self, name: str) -> Optional[Type[Plugin]]:
        """加载单个插件。

        Args:
            name: 插件名称，格式和 Python `import` 语句相同。

        Returns:
            被加载的插件类。
        """
        self._extend_plugins.append(name)
        return self._load_plugin(name)

    def _load_plugins_from_dir(self, path: Iterable[str]):
        """从指定路径列表中加载插件，以 `_` 开头的插件不会被导入。路径可以是相对路径或绝对路径。

        Args:
            path: 由储存插件的路径文本组成的列表。
                例如： `['path/of/plugins/', '/home/xxx/alicebot/plugins']` 。
        """
        self._module_path_finder.path.extend(path)
        for plugin_info in load_modules_from_dir(path, Plugin):
            try:
                self._load_plugin_module(plugin_info)
            except Exception as e:
                error_or_exception(
                    f'Load plugin "{plugin_info.module_class.__name__}" '
                    f'from file "{plugin_info.module.__file__}" failed:',
                    e,
                    self.config.verbose_exception_log,
                )
            else:
                logger.info(
                    f'Succeeded to load plugin "{plugin_info.module_class.__name__}" '
                    f'from file "{plugin_info.module.__file__}"'
                )

    def load_plugins_from_dir(self, path: Iterable[str]):
        """从指定路径列表中加载插件，以 `_` 开头的插件不会被导入。路径可以是相对路径或绝对路径。

        Args:
            path: 由储存插件的路径文本组成的列表。
                例如： `['path/of/plugins/', '/home/xxx/alicebot/plugins']` 。
        """
        self._extend_plugin_dirs.extend(path)
        self._load_plugins_from_dir(path)

    def _load_adapter(self, name: str) -> Optional[Adapter]:
        """加载单个适配器。

        Args:
            name: 适配器名称，格式和 Python `import` 语句相同。

        Returns:
            被加载的适配器对象。
        """
        try:
            adapter_info = load_module_from_name(name, Adapter)
            adapter_object = adapter_info.module_class(self)
        except Exception as e:
            error_or_exception(
                f'Load adapter "{name}" failed:', e, self.config.verbose_exception_log
            )
        else:
            self._adapters.append((adapter_object, adapter_info))
            self._update_config_module(adapter_info.config_class)
            logger.info(
                f'Succeeded to load adapter "{adapter_info.module_class.__name__}" '
                f'from module "{name}"'
            )
            return adapter_object

    def load_adapter(self, name: str) -> Optional[Adapter]:
        """加载单个适配器。

        Args:
            name: 适配器名称，格式和 Python `import` 语句相同。

        Returns:
            被加载的适配器对象。
        """
        self._extend_adapters.append(name)
        return self._load_adapter(name)

    def _update_config_module(self, config_class: Optional[Type[BaseModel]]):
        """更新配置模型。"""
        if config_class is None:
            return
        try:
            default_value = config_class()
        except ValidationError:
            default_value = ...
        self._config_update_dict[getattr(config_class, "__config_name__")] = (
            config_class,
            default_value,
        )

    def _remove_config_module(self, config_class: Optional[Type[BaseModel]]):
        """删除配置模型。"""
        if config_class is None:
            return
        try:
            self._config_update_dict.pop(getattr(config_class, "__config_name__"))
        except KeyError:
            pass

    def _reload_config(self):
        """更新 config，合并入来自 Plugin 和 Adapter 的 Config"""
        self.config = create_model(
            "Config", **self._config_update_dict, __base__=MainConfig
        )(**self._raw_config_dict)

    def get_loaded_adapter_by_name(self, name: str) -> Adapter:
        """按照名称获取已经加载的适配器。

        Args:
            name: 适配器名称。

        Returns:
            获取到的适配器对象。

        Raises:
            LookupError: 找不到此名称的适配器对象。
        """
        for _adapter, _ in self._adapters:
            if _adapter.name == name:
                return _adapter
        raise LookupError

    def bot_run_hook(self, func: T_BotHook) -> T_BotHook:
        """注册一个 Bot 启动时的函数。

        Args:
            func: 被注册的函数。

        Returns:
            被注册的函数。
        """
        self._bot_run_hooks.append(func)
        return func

    def bot_exit_hook(self, func: T_BotExitHook) -> T_BotExitHook:
        """注册一个 Bot 退出时的函数。

        Args:
            func: 被注册的函数。

        Returns:
            被注册的函数。
        """
        self._bot_exit_hooks.append(func)
        return func

    def adapter_startup_hook(self, func: T_AdapterHook) -> T_AdapterHook:
        """注册一个适配器初始化时的函数。

        Args:
            func: 被注册的函数。

        Returns:
            被注册的函数。
        """
        self._adapter_startup_hooks.append(func)
        return func

    def adapter_run_hook(self, func: T_AdapterHook) -> T_AdapterHook:
        """注册一个适配器运行时的函数。

        Args:
            func: 被注册的函数。

        Returns:
            被注册的函数。
        """
        self._adapter_run_hooks.append(func)
        return func

    def adapter_shutdown_hook(self, func: T_AdapterHook) -> T_AdapterHook:
        """注册一个适配器关闭时的函数。

        Args:
            func: 被注册的函数。

        Returns:
            被注册的函数。
        """
        self._adapter_shutdown_hooks.append(func)
        return func

    def event_preprocessor_hook(self, func: T_EventHook) -> T_EventHook:
        """注册一个事件预处理函数。

        Args:
            func: 被注册的函数。

        Returns:
            被注册的函数。
        """
        self._event_preprocessor_hooks.append(func)
        return func

    def event_postprocessor_hook(self, func: T_EventHook) -> T_EventHook:
        """注册一个事件后处理函数。

        Args:
            func: 被注册的函数。

        Returns:
            被注册的函数。
        """
        self._event_postprocessor_hooks.append(func)
        return func



"""AliceBot 日志。

AliceBot 使用 [loguru](https://github.com/Delgan/loguru) 来记录日志信息。
自定义 logger 请参考 [loguru](https://github.com/Delgan/loguru) 文档。
"""
from loguru import logger as _logger

logger = _logger


def error_or_exception(message: str, exception: Exception, verbose: bool):
    if verbose:
        logger.exception(message)
    else:
        logger.error(f"{message} {exception!r}")



"""AliceBot 插件。

所有 AliceBot 插件的基类。所有用户编写的插件必须继承自 `Plugin` 类。
"""
from abc import ABC, abstractmethod
from typing import TYPE_CHECKING, Generic, NoReturn

from alicebot.typing import T_Event, T_State
from alicebot.exceptions import SkipException, StopException

if TYPE_CHECKING:
    from alicebot import Bot
    from alicebot.config import MainConfig

__all__ = ["Plugin"]


class Plugin(ABC, Generic[T_Event, T_State]):
    """所有 AliceBot 插件的基类。

    Attributes:
        event: 当前正在被此插件处理的事件。
        priority: 插件的优先级，数字越小表示优先级越高，默认为 0。
        block: 插件执行结束后是否阻止事件的传播。True 表示阻止。
    """

    event: T_Event
    priority: int = 0
    block: bool = False

    def __init__(self, event: T_Event):
        self.event = event

        if not hasattr(self, "priority"):
            self.priority = 0
        if not hasattr(self, "block"):
            self.block = False

        self.get = self.bot.get

        self.__post_init__()

    def __post_init__(self):
        """用于初始化后处理，被 `__init__()` 方法调用。"""
        pass

    @property
    def name(self) -> str:
        """插件类名称。"""
        return self.__class__.__name__

    @property
    def bot(self) -> "Bot":
        """机器人对象。"""
        return self.event.adapter.bot

    @property
    def config(self) -> "MainConfig":
        """机器人配置。"""
        return self.bot.config

    def stop(self) -> NoReturn:
        """停止当前事件传播。"""
        raise StopException()

    def skip(self) -> NoReturn:
        """跳过自身继续当前事件传播。"""
        raise SkipException()

    @property
    def state(self) -> T_State:
        """插件状态。"""
        return self.bot.plugin_state[self.name]

    @state.setter
    def state(self, value: T_State):
        self.bot.plugin_state[self.name] = value

    @abstractmethod
    async def handle(self) -> None:
        """处理事件的方法。当 `rule()` 方法返回 `True` 时 AliceBot 会调用此方法。每个插件必须实现此方法。"""
        raise NotImplementedError

    @abstractmethod
    async def rule(self) -> bool:
        """匹配事件的方法。事件处理时，会按照插件的优先级依次调用此方法，当此方法返回 `True` 时将事件交由此插件处理。每个插件必须实现此方法。

        注意：不建议直接在此方法内实现对事件的处理，事件的具体处理请交由 `handle()` 方法。
        """
        raise NotImplementedError



"""AliceBot 消息。

实现了常用的基本消息 `Message` 和消息字段 `MessageSegment` 模型供适配器使用。
适配器开发者可以根据需要实现此模块中消息类的子类或定义与此不同的消息类型，但建议若可行的话应尽量使用此模块中消息类的子类。
"""
import dataclasses
from copy import copy, deepcopy
from dataclasses import field, dataclass
from typing import (
    Any,
    Dict,
    List,
    Type,
    Union,
    Generic,
    Mapping,
    TypeVar,
    Iterable,
    Iterator,
)

__all__ = ["T_Message", "T_MessageSegment", "T_MS", "Message", "MessageSegment"]

T_Message = TypeVar("T_Message", bound="Message")
T_MessageSegment = TypeVar("T_MessageSegment", bound="MessageSegment")

# 可以转化为 MessageSegment 的类型
T_MS = Union[T_MessageSegment, str, Mapping]


class Message(List[T_MessageSegment]):
    """消息。

    本类是 `List` 的子类，并重写了 `__init__()` 方法，
    可以直接处理 str, Mapping, Iterable[Mapping], T_MessageSegment, T_Message。
    其中 str 的支持需要适配器开发者重写 `_str_to_message_segment()` 方法实现。
    本类重写了 `+` 和 `+=` 运算符，可以直接和 Message, MessageSegment 等类型的对象执行取和运算。
    若开发者实现了 MessageSegment 的子类则需要重写 `_message_segment_class()` 方法，
    并在 MessageSegment 的子类中重写 `_message_class()` 方法。
    """

    def __init__(
        self,
        message: Union[None, T_Message, T_MS, Iterable[T_MS]] = None,
        *args,
        **kwargs,
    ):
        super().__init__(*args, **kwargs)
        if message is None:
            return
        elif isinstance(message, self.__class__):
            self.extend(message)
        else:
            self.extend(self._construct(message))

    @property
    def _message_segment_class(self) -> Type[T_MessageSegment]:
        """若开发者实现了 MessageSegment 的子类则需要重写此方法。

        Returns:
            MessageSegment 类。
        """
        return MessageSegment

    @classmethod
    def __get_validators__(cls):
        """pydantic 自定义校验器。"""
        yield cls._validate

    @classmethod
    def _validate(cls, value):
        return cls(value)

    def _construct(
        self, msg: Union[T_MS, Iterable[T_MS]]
    ) -> Iterator[T_MessageSegment]:
        """用于将 str, Mapping, Iterable[Mapping] 等类型转换为 MessageSegment。
        用于 pydantic 数据解析和方便用户使用。

        Args:
            msg: 要解析为 MessageSegment 的数据。

        Returns:
            MessageSegment 生成器。
        """
        if isinstance(msg, self._message_segment_class):
            yield msg
        elif isinstance(msg, Mapping):
            yield self._mapping_to_message_segment(msg)
        elif isinstance(msg, str):
            yield self._str_to_message_segment(msg)
        elif isinstance(msg, Iterable):
            for seg in msg:
                for i in self._construct(seg):
                    yield i

    def _mapping_to_message_segment(self, msg: Mapping) -> T_MessageSegment:
        """用于将 Mapping 转换为 MessageSegment，如有需要，子类可重写此方法以更改对 Mapping 的默认行为。

        Args:
            msg: 要解析为 MessageSegment 的数据。

        Returns:
            由 Mapping 转换的 MessageSegment。
        """
        return self._message_segment_class(**msg)

    def _str_to_message_segment(self, msg: str) -> T_MessageSegment:
        """用于将 str 转换为 MessageSegment，子类应重写此方法以支持 str 及支持新的消息字段类。

        Args:
            msg: 要解析为 MessageSegment 的数据。

        Returns:
            由 str 转换的 MessageSegment。
        """
        raise NotImplementedError

    def __repr__(self) -> str:
        return "Message:[{}]".format(",".join(map(lambda x: repr(x), self)))

    def __str__(self) -> str:
        return "".join(map(lambda x: str(x), self))

    def __contains__(self, item) -> bool:
        if isinstance(item, str):
            return item in str(self)
        return super().__contains__(item)

    def __add__(
        self: T_Message, other: Union[T_Message, T_MessageSegment, str]
    ) -> T_Message:
        return self.__class__(self).__iadd__(other)

    def __radd__(
        self: T_Message, other: Union[T_Message, T_MessageSegment, str]
    ) -> T_Message:
        return self.__class__(other).__iadd__(self)

    def __iadd__(
        self: T_Message, other: Union[T_Message, T_MessageSegment, str]
    ) -> T_Message:
        if isinstance(other, type(self)):
            self.extend(other)
        if isinstance(other, self._message_segment_class):
            self.append(other)
        elif isinstance(other, str):
            self.extend(self._construct(other))
        else:
            raise TypeError(
                f"unsupported operand type(s) for +: '{type(self)}' and '{type(other)}'"
            )
        return self

    def is_text(self) -> bool:
        """
        Returns:
            是否是纯文本消息。
        """
        return all(map(lambda x: x.is_text(), self))

    def get_plain_text(self) -> str:
        """获取消息中的纯文本部分。

        Returns:
            消息中的纯文本部分。
        """
        return "".join(map(lambda x: str(x), filter(lambda x: x.is_text(), self)))

    def copy(self) -> T_Message:
        """返回自身的浅复制。

        Returns:
            自身的浅复制。
        """
        return self.__class__(self)

    def deepcopy(self) -> T_Message:
        """返回自身的深复制。

        Returns:
            自身的深复制。
        """
        return deepcopy(self)

    def startswith(
        self, prefix: Union[str, T_MessageSegment], start=None, end=None
    ) -> bool:
        """实现类似字符串的 `startswith()` 方法。

        当 `prefix` 类型是 str 时，会将自身转换为 str 类型，再调用 str 类型的 `startswith()` 方法。
        当 `prefix` 类型是 T_MessageSegment 时，`start` 和 `end` 参数将不其作用，
            会判断列表的第一个消息字段是否和 `prefix` 相等。

        Args:
            prefix: 前缀。
            start: 开始检查位置。
            end: 停止检查位置。

        Returns:
            检查结果。
        """
        if isinstance(prefix, str):
            return str(self).startswith(prefix, start, end)
        elif isinstance(prefix, self._message_segment_class):
            if len(self) == 0:
                return False
            return self[0] == prefix
        raise TypeError(
            f"first arg must be str or {self._message_segment_class}，not {type(prefix)}"
        )

    def endswith(
        self, suffix: Union[str, T_MessageSegment], start=None, end=None
    ) -> bool:
        """实现类似字符串的 `endswith()` 方法。

        当 `suffix` 类型是 str 时，会将自身转换为 str 类型，再调用 str 类型的 `endswith()` 方法。
        当 `suffix` 类型是 T_MessageSegment 时，`start` 和 `end` 参数将不其作用，
            会判断列表的最后一个消息字段是否和 `suffix` 相等。

        Args:
            suffix: 后缀。
            start: 开始检查位置。
            end: 停止检查位置。

        Returns:
            检查结果。
        """
        if isinstance(suffix, str):
            return str(self).endswith(suffix, start, end)
        elif isinstance(suffix, self._message_segment_class):
            if len(self) == 0:
                return False
            return self[-1] == suffix
        raise TypeError(
            f"first arg must be str or {self._message_segment_class}，not {type(suffix)}"
        )

    def replace(
        self,
        old: Union[str, T_MessageSegment],
        new: Union[str, T_MessageSegment, None],
        count: int = -1,
    ) -> T_Message:
        """实现类似字符串的 `replace()` 方法。

        当 `old` 为 str 类型时，`new` 也必须是 str ，本方法将仅对 `is_text()` 为 True 的消息字段进行处理。
        当 `old` 为 MessageSegment 类型时，`new` 可以是 MessageSegment 或 None，本方法将对所有消息字段进行处理，
            并替换符合条件的消息字段。None 表示删除匹配到的消息字段。

        Args:
            old: 被匹配的字符串或消息字段。
            new: 被替换为的字符串或消息字段。
            count: 替换的次数。

        Returns:
            替换后的消息对象。
        """
        if type(old) == str:
            if type(new) != str:
                raise TypeError("when type of old is str, type of new must be str.")
            return self._replace_str(old, new, count)
        elif isinstance(old, self._message_segment_class):
            if not (isinstance(new, self._message_segment_class) or new is None):
                raise TypeError(
                    "when type of old is MessageSegment, "
                    "type of new must be MessageSegment or None."
                )
            temp_msg = self.deepcopy()
            for index, item in enumerate(temp_msg):
                if count == 0:
                    break
                if item == old:
                    temp_msg[index] = new
                    count -= 1
            if new is None:
                temp_msg = self.__class__(filter(lambda x: x is not None, self))
            return temp_msg
        else:
            raise TypeError("type of old must be str or MessageSegment")

    def _replace_str(self, old: str, new: str, count: int = -1) -> T_Message:
        """实现类似字符串的 `replace()` 方法。

        本方法将被 `replace()` 方法调用以处理 str 类型的替换，
        默认将 MessageSegment 对象的 data['text'] 视为存放纯文本的位置。
        适配器开发者可自行重写此方法以适配其他情况。

        Args:
            old: 被匹配的字符串或消息字段。
            new: 被替换为的字符串或消息字段。
            count: 替换的次数。

        Returns:
            替换后的消息对象。
        """
        temp_msg = self.deepcopy()
        for index, item in enumerate(temp_msg):
            item: MessageSegment
            if count == 0:
                break
            if item.is_text() and old in item.data["text"]:
                if count == -1:
                    temp_msg[index].data["text"] = item.data["text"].replace(old, new)
                else:
                    replace_times = min(count, item.data["text"].count(old))
                    temp_msg[index].data["text"] = item.data["text"].replace(
                        old, new, replace_times
                    )
                    count -= replace_times
        return temp_msg


@dataclass
class MessageSegment(Mapping, Generic[T_Message]):
    """消息字段。

    本类实现了所有映射类型的方法，这些方法全部是对 `data` 属性的操作。
    本类重写了 `+` 和 `+=` 运算符，可以直接和 Message, MessageSegment 等类型的对象执行取和运算，返回 Message 对象。
    若开发者实现了 Message 和 MessageSegment 的子类则需要重写 `_message_class()` 方法。

    Attributes:
        type: 消息字段类型。
        data: 消息字段内容。
    """

    type: str
    data: Dict[str, Any] = field(default_factory=dict)

    @property
    def _message_class(self) -> Type[T_Message]:
        """若开发者实现了 Message 和 MessageSegment 的子类则需要重写此方法。

        Returns:
            Message 类。
        """
        return Message

    def as_dict(self) -> Dict[str, Any]:
        """将当前对象解析为 Dict 对象，开发者可重写本方法以适配特殊的解析方式。

        Returns:
            Dict 对象。
        """
        return dataclasses.asdict(self)

    def __str__(self) -> str:
        return str(self.data)

    def __repr__(self) -> str:
        return f"MessageSegment<{self.type}>:{str(self)}"

    def __getitem__(self, key) -> Any:
        return self.data[key]

    def __setitem__(self, key, value):
        self.data[key] = value

    def __delitem__(self, key):
        del self.data[key]

    def __len__(self) -> int:
        return len(self.data)

    def __iter__(self):
        yield from self.data.__iter__()

    def __contains__(self, key) -> bool:
        return key in self.data

    def __eq__(self, other: T_MessageSegment) -> bool:
        return self.type == other.type and self.data == other.data

    def __ne__(self, other: T_MessageSegment) -> bool:
        return not self.__eq__(other)

    def __add__(self, other) -> T_Message:
        return self._message_class(self) + other

    def __radd__(self, other) -> T_Message:
        return self._message_class(other) + self

    def get(self, key: str, default=None):
        return self.data.get(key, default)

    def keys(self):
        return self.data.keys()

    def values(self):
        return self.data.values()

    def items(self):
        return self.data.items()

    def is_text(self) -> bool:
        """
        Returns:
            是否是纯文本消息字段。
        """
        return self.type == "text"

    def copy(self) -> T_MessageSegment:
        """返回自身的浅复制。

        Returns:
            自身的浅复制。
        """
        return copy(self)

    def deepcopy(self) -> T_MessageSegment:
        """返回自身的深复制。

        Returns:
            自身的深复制。
        """
        return deepcopy(self)



"""HTTP 服务端适配器示例。

这里是一个最简单可以直接使用的 HTTP 服务端适配器示例。
"""
from typing import TYPE_CHECKING, Union

from aiohttp import web

from alicebot.event import Event
from alicebot.adapter.utils import HttpServerAdapter
from alicebot.message import Message, MessageSegment

if TYPE_CHECKING:
    from alicebot.message import T_Message, T_MessageSegment


class HttpServerTestEvent(Event["HttpServerTestAdapter"]):
    """HTTP 服务端示例适配器事件类。"""

    message: Message


class HttpServerTestAdapter(HttpServerAdapter):
    """HTTP 服务端示例适配器类。"""

    get_url: str = "/"
    post_url: str = "/"
    host: str = "127.0.0.1"
    port: int = 8080

    async def handle_response(self, request: web.Request):
        event = HttpServerTestEvent(
            adapter=self,
            type="message",
            message=Message(
                MessageSegment(type="text", data={"text": await request.text()})
            ),
        )
        await self.handle_event(event)
        return web.Response()

    @staticmethod
    async def send(msg: Union[str, "T_Message", "T_MessageSegment"]):
        print(msg)



"""AliceBot 事件。

事件类的基类。适配器开发者应实现此事件类基类的子类。
"""
from abc import ABC
from typing import Any, Generic, Optional

from pydantic import BaseModel, PrivateAttr

from alicebot.message import Message
from alicebot.typing import T_Adapter
from alicebot.utils import DataclassEncoder

__all__ = ["Event"]


class Event(ABC, BaseModel, Generic[T_Adapter]):
    """事件类的基类。

    Attributes:
        adapter: 产生当前事件的适配器对象。
        type: 事件类型。
        __handled__: 表示事件是否被处理过了，用于适配器处理。
            警告：请勿手动更改此属性的值。
    """

    adapter: Any
    # 这里的 adapter 类型定义只是为了 IDE 的类型检查工具能够正常工作，这个字段将永远不会被实际使用
    # IDE 对 BaseModel 实例化时的提示会将 BaseModel 视为 dataclasses，而忽略 __init__ 定义
    # 因此需要此定义防止类型提示出错
    # 参考：
    # https://pydantic-docs.helpmanual.io/visual_studio_code/#technical-details
    # https://koxudaxi.github.io/pydantic-pycharm-plugin/ignore-init-arguments/

    type: Optional[str]

    _adapter: Optional[T_Adapter] = PrivateAttr()  # adapter 实际上放在这里
    __handled__: bool = PrivateAttr(default=False)

    def __init__(self, **data):
        if "adapter" in data:
            self._adapter = data.pop("adapter")
        else:
            raise ValueError('"adapter" argument must be set')
        super().__init__(**data)

    @property
    def adapter(self) -> T_Adapter:
        """产生当前事件的适配器对象。"""
        return self._adapter

    def __str__(self) -> str:
        return f"Event<{self.type}>"

    def __repr__(self) -> str:
        return self.__str__()

    class Config:
        extra = "allow"
        json_encoders = {Message: DataclassEncoder}



"""适配器实用工具。

这里定义了一些在编写适配器时常用的基类，适配器开发者可以直接继承自这里的类或者用作参考。
"""
import asyncio
from typing import Union, Literal
from abc import ABCMeta, abstractmethod

import aiohttp
from aiohttp import web

from alicebot.adapter import Adapter
from alicebot.log import logger, error_or_exception


class PollingAdapter(Adapter, metaclass=ABCMeta):
    """轮询式适配器示例。"""

    delay: Union[int, float] = 0.1
    create_task: bool = False

    async def run(self):
        while not self.bot.should_exit.is_set():
            await asyncio.sleep(self.delay)
            if self.create_task:
                asyncio.create_task(self.on_tick())
            else:
                await self.on_tick()

    @abstractmethod
    async def on_tick(self):
        pass


class HttpClientAdapter(PollingAdapter, metaclass=ABCMeta):
    """HTTP 客户端适配器示例。"""

    session: aiohttp.ClientSession

    async def startup(self):
        self.session = aiohttp.ClientSession()

    @abstractmethod
    async def on_tick(self):
        pass

    async def shutdown(self):
        await self.session.close()


class WebSocketClientAdapter(Adapter, metaclass=ABCMeta):
    """WebSocket 客户端适配器示例。"""

    url: str

    async def run(self):
        async with aiohttp.ClientSession() as session:
            async with session.ws_connect(self.url) as ws:
                msg: aiohttp.WSMessage
                async for msg in ws:
                    if self.bot.should_exit.is_set():
                        break
                    if msg.type == aiohttp.WSMsgType.ERROR:
                        break
                    await self.handle_response(msg)

    @abstractmethod
    def handle_response(self, msg: aiohttp.WSMessage):
        pass


class HttpServerAdapter(Adapter, metaclass=ABCMeta):
    """HTTP 服务端适配器示例。"""

    app: web.Application
    runner: web.AppRunner
    site: web.TCPSite
    host: str
    port: int
    get_url: str
    post_url: str

    async def startup(self):
        self.app = web.Application()
        self.app.add_routes(
            [
                web.get(self.get_url, self.handle_response),
                web.post(self.post_url, self.handle_response),
            ]
        )

    async def run(self):
        self.runner = web.AppRunner(self.app)
        await self.runner.setup()
        self.site = web.TCPSite(self.runner, self.host, self.port)
        await self.site.start()

    async def shutdown(self):
        await self.runner.cleanup()

    @abstractmethod
    async def handle_response(self, request: web.Request):
        pass


class WebSocketServerAdapter(Adapter, metaclass=ABCMeta):
    """WebSocket 服务端适配器示例。"""

    app: web.Application = None
    runner: web.AppRunner = None
    site: web.TCPSite = None
    websocket: web.WebSocketResponse = None
    host: str
    port: int
    url: str

    async def startup(self):
        self.app = web.Application()
        self.app.add_routes([web.get(self.url, self.handle_response)])

    async def run(self):
        self.runner = web.AppRunner(self.app)
        await self.runner.setup()
        self.site = web.TCPSite(self.runner, self.host, self.port)
        await self.site.start()

    async def shutdown(self):
        if self.websocket is not None:
            await self.websocket.close()
        if self.site is not None:
            await self.site.stop()
        if self.runner is not None:
            await self.runner.cleanup()

    async def handle_response(self, request: web.Request):
        ws = web.WebSocketResponse()
        await ws.prepare(request)
        self.websocket = ws

        msg: aiohttp.WSMessage
        async for msg in ws:
            if msg.type == aiohttp.WSMsgType.TEXT:
                await self.handle_ws_response(msg)
            elif msg.type == aiohttp.WSMsgType.ERROR:
                break

        return ws

    @abstractmethod
    async def handle_ws_response(self, msg: aiohttp.WSMessage):
        pass


class WebSocketAdapter(Adapter, metaclass=ABCMeta):
    """WebSocket 适配器示例。

    同时支持 WebSocket 客户端和服务端。
    """

    websocket: Union[web.WebSocketResponse, aiohttp.ClientWebSocketResponse] = None

    # ws
    session: aiohttp.ClientSession = None

    # reverse-ws
    app: web.Application = None
    runner: web.AppRunner = None
    site: web.TCPSite = None

    # config
    adapter_type: Literal["ws", "reverse-ws"]
    host: str
    port: int
    url: str
    reconnect_interval: int = 3

    async def startup(self):
        """初始化适配器。"""
        if self.adapter_type == "ws":
            self.session = aiohttp.ClientSession()
        elif self.adapter_type == "reverse-ws":
            self.app = web.Application()
            self.app.add_routes([web.get(self.url, self.handle_reverse_ws_response)])
        else:
            logger.error(
                'Config "adapter_type" must be "ws" or "reverse-ws", not '
                + self.adapter_type
            )

    async def run(self):
        """运行适配器。"""
        if self.adapter_type == "ws":
            while True:
                try:
                    await self.websocket_connect()
                except aiohttp.ClientError as e:
                    error_or_exception(
                        "WebSocket connection error:",
                        e,
                        self.bot.config.verbose_exception_log,
                    )
                if self.bot.should_exit.is_set():
                    break
                await asyncio.sleep(self.reconnect_interval)
        elif self.adapter_type == "reverse-ws":
            self.runner = web.AppRunner(self.app)
            await self.runner.setup()
            self.site = web.TCPSite(self.runner, self.host, self.port)
            await self.site.start()

    async def shutdown(self):
        """关闭并清理连接。"""
        if self.websocket is not None:
            await self.websocket.close()
        if self.adapter_type == "ws":
            if self.session is not None:
                await self.session.close()
        elif self.adapter_type == "reverse-ws":
            if self.site is not None:
                await self.site.stop()
            if self.runner is not None:
                await self.runner.cleanup()

    async def handle_reverse_ws_response(self, request: web.Request):
        """处理 aiohttp WebSocket 服务器的接收。"""
        self.websocket = web.WebSocketResponse()
        await self.websocket.prepare(request)
        await self.reverse_ws_connection_hook()
        await self.handle_websocket()
        return self.websocket

    async def reverse_ws_connection_hook(self):
        """反向 WebSocket 连接建立时的钩子函数。"""
        logger.info(f"WebSocket connected!")

    async def websocket_connect(self):
        """创建正向 WebSocket 连接。"""
        logger.info("Tying to connect to WebSocket server...")
        async with self.session.ws_connect(
            f"ws://{self.host}:{self.port}{self.url}"
        ) as self.websocket:
            await self.handle_websocket()

    async def handle_websocket(self):
        """处理 WebSocket。"""
        if self.websocket.closed:
            return
        async for msg in self.websocket:
            msg: aiohttp.WSMessage
            await self.handle_websocket_msg(msg)
        if not self.bot.should_exit.is_set():
            logger.warning("WebSocket connection closed!")

    @abstractmethod
    async def handle_websocket_msg(self, msg: aiohttp.WSMessage):
        """处理 WebSocket 消息。"""
        raise NotImplementedError
